#Region "Autodesk Copyright" 

' .NET Sample 
' 
' Copyright (c) 2008 by Autodesk, Inc. 
' 
' Permission to use, copy, modify, and distribute this software 
' for any purpose and without fee is hereby granted, provided 
' that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and 
' restricted rights notice below appear in all supporting 
' documentation. 
' 
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
' UNINTERRUPTED OR ERROR FREE. 
' 
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer 
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
' (Rights in Technical Data and Computer Software), as applicable. 
' 

#End Region 

#Region "Namespaces References" 

Imports System 
Imports System.Collections.Generic 
Imports System.Text 
Imports System.Diagnostics 

' AutoCAD MEP Specific 
' 
Imports Autodesk.AutoCAD.DatabaseServices 
Imports Autodesk.AutoCAD.Runtime 
Imports Autodesk.AutoCAD.Geometry 
Imports Autodesk.AutoCAD.ApplicationServices 
Imports Autodesk.AutoCAD.EditorInput 
Imports Autodesk.AutoCAD.GraphicsSystem 
Imports Autodesk.Aec.DatabaseServices 
Imports Autodesk.Aec.Building.ApplicationServices 
Imports Autodesk.Aec.Building.DatabaseServices 
Imports Autodesk.Aec.Building.Hvac.DatabaseServices 

' create aliases 
' 
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports Entity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports DictionaryProfileDefinition = Autodesk.Aec.Building.DatabaseServices.DictionaryProfileDefinition
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AddAirTerminal))> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(AddAirTerminal))> 

Public Class AddAirTerminal
    Implements IExtensionApplication

    Private mEditor As Editor
    Private mDb As Database
    Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager
    Private mTrans As Transaction

#Region "IExtensionApplication"
    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading AddAirTerminal..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub
#End Region

#Region "CmdAddAirTerminal"
    <CommandMethod("AddAirTerminal", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub CmdAddAirTermial()
        mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
        mTransMgr = AcadApplication.DocumentManager.MdiActiveDocument.TransactionManager

        ' start the transaction 
        mTrans = mTransMgr.StartTransaction()

        Try
            ' get the query object 
            Dim query As DataQuery = GetQueryObject()

            ' get the part list of diffusers based on the query 
            Dim partList As DataPartsList = PartManager.GetPartsList(query)

            ' get a specific diffuser from the user 
            Dim result As PromptIntegerResult = PromptForSelection(partList)

            If result.Status <> PromptStatus.OK Then
                Return
            End If

            ' get the part lookup item for the specific part 
            Dim dpl As DataPartLookup = partList.DataParts(result.Value)

            ' assign the selected part lookup id to query's part part id 
            query.PartGuid = dpl.Guid

            ' create the part 
            Dim mvPart As MultiViewPart = CreateDiffuser(query)

            Dim ppr As PromptPointResult = mEditor.GetPoint("" & Chr(10) & "Specify insert point: ")

            If ppr.Status <> PromptStatus.OK Then
                Return
            End If

            ' assign selected point
            mvPart.Location = ppr.Value

            ' set the layer 
            mvPart.SetDefaultLayer()

            ' prompt for and set the system for the part 
            mvPart.SystemId = GetSystem()

            ' finally we will add the part to the database and commit the transaction 
            Dim bt As BlockTable = DirectCast(mTransMgr.GetObject(mDb.BlockTableId, OpenMode.ForWrite, False), BlockTable)
            Dim btr As BlockTableRecord = DirectCast(mTransMgr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite, False), BlockTableRecord)
            btr.AppendEntity(mvPart)

            mTransMgr.AddNewlyCreatedDBObject(mvPart, True)
        Catch ex As System.Exception
            mEditor.WriteMessage(ex.Message)
        Finally
            mTrans.Commit()
        End Try
    End Sub
#End Region

#Region "CreateDiffuser"
    ''' <summary> 
    ''' Creates the diffuser object and assigns the part data 
    ''' </summary> 
    ''' <param name="query">represents the data query</param> 
    ''' <returns>MultiViewPart</returns> 
    Private Function CreateDiffuser(ByVal query As DataQuery) As MultiViewPart
        ' get the expanded table from the query 
        Dim exTbl As DataExpandedTable = PartManager.GetPartTable(query, 100)

        ' create the new mvpart 
        Dim mvPart As New MultiViewPart()
        mvPart.SetDatabaseDefaults()
        mvPart.SetToStandard(mDb)
        PartManager.CreatePartViaRecord(exTbl, exTbl.DataRecords(0), mvPart)
        Return mvPart
    End Function
#End Region


#Region "PromptForSelection"

    ''' <summary> 
    ''' returns the index of the selected part 
    ''' </summary> 
    ''' <param name="partList"></param> 
    ''' <returns>PromptIntegerResult</returns> 
    Private Function PromptForSelection(ByVal partList As DataPartsList) As PromptIntegerResult
        For i As Integer = 1 To partList.DataParts.Count
            mEditor.WriteMessage("" & Chr(10) & "" + i.ToString() + ") " + partList.DataParts(i - 1).Name)
        Next
        Dim intOps As New PromptIntegerOptions("" & Chr(10) & "Select part: ")

        ' set the prompt options 
        intOps.AllowArbitraryInput = False
        intOps.AllowNegative = False
        intOps.AllowNone = False
        intOps.LowerLimit = 1

        ' set the upper limit of the integer prompt to the number of parts found 
        intOps.UpperLimit = partList.DataParts.Count - 1

        ' issue the prompt and return the result 
        Return mEditor.GetInteger(intOps)
    End Function
#End Region


#Region "GetQueryObject"

    ''' <summary> 
    ''' Gets a DataQuery object set up for a diffuser 
    ''' </summary> 
    ''' <returns>DataQuery</returns> 
    Private Shared Function GetQueryObject() As DataQuery
        ' create a new data query object 
        Dim query As New DataQuery()

        'assign it the mvpart domain 
        query.Domain = Domain.MultiViewPartComponent

        'specify the type to be an air terminal 
        query.Type = Autodesk.Aec.Building.DatabaseServices.Type.AirTerminal

        ' return the new query object 
        Return query
    End Function
#End Region

#Region "GetSystem"
    ''' <summary> 
    ''' Gets a system id 
    ''' </summary> 
    ''' <returns>ObjectId of the system selected</returns> 
    Private Function GetSystem() As ObjectId
        Dim dict As New DictionaryDuctSystemDefinition(mDb)

        Dim oids As ObjectId() = New ObjectId(dict.Records.Count - 1) {}
        For i As Integer = 0 To dict.Records.Count - 1

            ' get the record at the current index 
            Dim dsd As SystemDefinition = DirectCast(mTransMgr.GetObject(dict.Records(i), OpenMode.ForRead, False), SystemDefinition)
            mEditor.WriteMessage("" & Chr(10) & "" + (i + 1).ToString() + ") " + dsd.Name)
            oids(i) = dict.Records(i)
        Next

        Dim intOps As New PromptIntegerOptions("" & Chr(10) & "Select system: ")

        ' set the prompt options 
        intOps.AllowArbitraryInput = False
        intOps.AllowNegative = False
        intOps.AllowNone = False
        intOps.LowerLimit = 1

        ' set the upper limit of the integer prompt to the number of parts found 
        intOps.UpperLimit = dict.Records.Count - 1

        ' issue the prompt and return the result 
        Dim result As PromptIntegerResult = mEditor.GetInteger(intOps)
        If result.Status <> PromptStatus.OK Then
            Return ObjectId.Null
        End If

        Return oids(result.Value - 1)
    End Function
#End Region
End Class