#Region " Autodesk Copyright"
'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'
#End Region

#Region "Namespaces References"
Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports TransactionManager = Autodesk.AutoCAD.DatabaseServices.TransactionManager
Imports PartType = Autodesk.Aec.Building.DatabaseServices.Type
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(AddToCatalog.AddToCatalog))> 

Namespace AddToCatalog
    Public Class AddToCatalog

#Region "Member Data"

        Private mEditor As Editor
        Private mDb As Database
        Private mDoc As Document
        Private mTransMgr As TransactionManager
        Private mCatalog As CatalogManager

#End Region

#Region "Helper Setup"

        Protected Sub Setup()
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mDoc = AcadApplication.DocumentManager.MdiActiveDocument
            mTransMgr = mDb.TransactionManager
            mCatalog = New CatalogManager
        End Sub

#End Region

#Region "Command AddCustomData"

        <Autodesk.AutoCAD.Runtime.CommandMethod("CatalogApiSample", "AddCustomData", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub AddCustomData()
            Try
                Setup()
                ' We need to create a part lookup object prior to loading the raw table of the part.  
                ' We will query the duct catalog for a round duct.  The catalog being queried
                ' can be specified through the options dialog.

                Dim query As DataQuery = New DataQuery

                query.Domain = Domain.DuctComponent 'Set the domain on the query
                query.Type = PartType.Duct ' Set the part type - in this case a duct
                query.SetPortShape(1, BuiltInShape.Circular) ' Set the port shape

                Dim partslist As DataPartsList = mCatalog.GetPartsList(query)

                If partslist.DataParts.Count > 0 Then ' Check that we we got at least 1 result - we may have more

                    Dim dpl As DataPartLookup
                    dpl = partslist.DataParts(0)

                    If dpl <> Nothing Then

                        ' Now, that we have the part lookup object, we can get the raw table of the part.  The raw table
                        ' represents a run time version of the xml data for the part.
                        '
                        Dim rawTable As DataRawTable = mCatalog.GetRawPartTable(dpl)

                        ' get the collection of ColumnConsts from the raw table
                        '
                        Dim dfc As DataFieldCollection = rawTable.ConstSet

                        ' Get the next available index from the raw table.
                        '
                        Dim index As Integer
                        index = rawTable.GetNextFreeParameterIndex(Context.CustomData)

                        ' Create our new DataField that we will add to the raw table
                        '
                        Dim field As DataField = New DataField(Context.CustomData, index)

                        ' Set the properties on the DataField.  In this sample, we will set the name, description
                        ' and an integer value.
                        '
                        Dim sIndex As String = field.Index.ToString

                        field.Name = "CD" + sIndex

                        field.Description = "CustomData" + sIndex

                        field.ValueInteger = 100

                        ' Now add the data field created to the raw table.
                        '
                        rawTable.ConstSet.Add(field)

                        ' Save the raw table to write the changes we have made back to the xml file and update the Catalog  
                        '
                        mCatalog.AddRawPartTable(dpl.Domain, dpl, rawTable, True)

                    End If
                End If

            Catch ex As System.Exception

                Debug.Write(String.Format("" & Microsoft.VisualBasic.Chr(10) & "Caught the following Exception:{0}", ex.ToString))

            End Try

        End Sub

#End Region

#Region "Command AddNewPart"

        <Autodesk.AutoCAD.Runtime.CommandMethod("CatalogApiSample", "AddNewPart", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub AddNewPart()

            ' This command will add a completely new part into the APC catalog with associated xml file. 
            ' The new part is based on an existing round duct, and is used to create the new part with some new/different values.
            '
            Try
                Setup()
                ' We need to create a part lookup object prior to loading the raw table of the part.  
                ' We will query the duct catalog for a round duct.  The catalog being queried
                ' can be specified through the options dialog.

                Dim query As DataQuery = New DataQuery

                query.Domain = Domain.DuctComponent 'Set the domain on the query
                query.Type = PartType.Duct ' Set the part type - in this case a duct
                query.SetPortShape(1, BuiltInShape.Circular) ' Set the port shape

                Dim partslist As DataPartsList = mCatalog.GetPartsList(query)

                If partslist.DataParts.Count > 0 Then ' Check that we we got at least 1 result - we may have more

                    Dim dpl As DataPartLookup
                    dpl = partslist.DataParts(0)

                    If dpl <> Nothing Then

                        ' Now, that we have the part lookup object, we can get the raw table of the part.  The raw table
                        ' represents a run time version of the xml data for the part.
                        '
                        Dim rawTable As DataRawTable = mCatalog.GetRawPartTable(dpl)

                        ' We are going to provide a different size list for our new part

                        ' First get the DataList for the existing sizes
                        Dim dl As DataList = rawTable.FindConstListByContext(Context.ConnectionSizeDiameter)

                        ' We will create a new DataField for to store new sizes based on 
                        ' context and index of first one in the current data list
                        Dim newSizeContext As Context = dl.DataFields(0).Context
                        Dim newSizeIndex As Integer = dl.DataFields(0).Index

                        'now delete all the existing sizes so we can add back our specialized sizes
                        dl.DataFields.Clear()

                        ' use the new size datafield to set new values into the data list
                        ' Adding back into the datalist 10 specialized sizes for the new part
                        Dim size As Double = 100
                        Dim newSizeDf As DataField
                        For i As Integer = 1 To 10
                            newSizeDf = New DataField(newSizeContext, newSizeIndex)
                            size = size + 10
                            newSizeDf.ValueDouble = size
                            dl.DataFields.Add(newSizeDf)
                        Next i


                        ' set up some values to use for the new part entry.
                        ' For GUID, get a new one so we know it is unique
                        Dim newPartId As String = Guid.NewGuid().ToString().ToUpper()

                        ' use GUID in name so we see it differently each time sample code is run.
                        Dim newPartName As String = "Round Duct Customized - " + newPartId

                        ' partname is also used in xml path, but does not have to match...
                        Dim newXmlPath As String = "Round\\Ducts\\" + newPartName + ".xml"

                        ' Description; sample was tested using US Imperial...
                        Dim newDescription As String = "Round Duct US Imperial Customized - " + newPartId

                        ' Set new part id in raw table
                        rawTable.PartId = newPartId
                        ' Set new part name in raw table
                        rawTable.PartName = newPartName
                        ' Set new part description in raw table
                        rawTable.PartDescription = newDescription

                        ' these are also necessary when you have new BMP and/or DWG... but not for this example.
                        'rawTable.ImageUrl = "New Image File if appropriate";
                        'rawTable.RecipePath = "New DWG file if appropriate";


                        ' so we know what new one was added, output the name...
                        mEditor.WriteMessage("\n New Part description is: " + rawTable.PartDescription)


                        ' we also need to update the data parts list
                        dpl.Guid = newPartId
                        ' need a new path and file name. This will be the new XML file written.
                        dpl.Path = newXmlPath
                        ' this is the new part name
                        dpl.Name = newPartName
                        ' for consistency sake we should also match this to be the same as the rawtable.PartDescription
                        dpl.Description = newDescription

                        ' Save the raw table to write the changes we have made back to the xml file and update the Catalog 
                        mCatalog.AddRawPartTable(dpl.Domain, dpl, rawTable, True)

                        ' This updates the APC file directly with the new entry...
                        mCatalog.AddCatalogPartEntry(dpl.Domain, newXmlPath, rawTable)

                        ' this will rebuild the entire APC file. For this small example it is not necessary because 
                        ' we added the part directly to the catalog in above call, but just know it is also avialable. 
                        'mCatalog.RegenerateCatalog(Domain.DuctComponent, true);

                    End If
                End If

            Catch ex As System.Exception

                Debug.Write(String.Format("" & Microsoft.VisualBasic.Chr(10) & "Caught the following Exception:{0}", ex.ToString))

            End Try

        End Sub

#End Region

    End Class
End Namespace

