#Region "Autodesk Copyright"

' .NET Sample 
' 
' Copyright (c) 2008, 2009 by Autodesk, Inc.
' 
' Permission to use, copy, modify, and distribute this software 
' for any purpose and without fee is hereby granted, provided 
' that the above copyright notice appears in all copies and 
' that both that copyright notice and the limited warranty and 
' restricted rights notice below appear in all supporting 
' documentation. 
' 
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. AUTODESK, INC. 
' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
' UNINTERRUPTED OR ERROR FREE. 
' 
' Use, duplication, or disclosure by the U.S. Government is subject to 
' restrictions set forth in FAR 52.227-19 (Commercial Computer 
' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
' (Rights in Technical Data and Computer Software), as applicable. 
' 

#End Region

#Region "Namespaces References"

' These imports allow us to use types in a namespace later in the code
' without having to enter the entire qualifying string each time we use a type.
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Elec.DatabaseServices

' Create aliases for certain namespaces/types for use later in the code.
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
Imports MepType = Autodesk.Aec.Building.DatabaseServices.Type

#End Region

#Region "Assemblies"

' Create assembly attributes to avoid scanning all public types (otherwise, you'll assert when loading the module).
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(DeviceStyles.CreateDeviceStyle))> 
<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(Nothing)> 

#End Region

' Our namespace.
Namespace DeviceStyles

    Public Class CreateDeviceStyle

#Region "Member Data"

        ' For our command class, we have a database member and a transaction manager member.
        ' Note: We're using the full qualifying string for the transaction manager (even though
        ' we have an active 'using' directive for this) because otherwise it would be an
        ' ambiguous reference between the 'Autodesk.AutoCAD.DatabaseServices.TransactionManager'
        ' and 'Autodesk.AutoCAD.ApplicationServices.TransactionManager' directives.

        Private mEditor As Editor
        Private mDb As Database
        Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager

#End Region

#Region "Setup"

        Protected Sub Setup()
            mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
            mDb = AcadApplication.DocumentManager.MdiActiveDocument.Database
            mTransMgr = mDb.TransactionManager
        End Sub

#End Region

#Region "Command CreateNewDeviceStyle"

        ' Here's our "CreateNewDeviceStyle" command; after the module is loaded,
        ' the user can enter CreateNewDeviceStyle on the commandline to access this command.
        <Autodesk.AutoCAD.Runtime.CommandMethod("CreateNewDeviceStyle", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
        Public Sub CmdCreateNewDeviceStyle()
            ' Init the data members, create the style, set some parameters in it.
            ' We catch any system exceptions and write out the corresponding error message.
            Try
                Setup()                                     ' Initialize the data members.
                Dim oidStyle As ObjectId = AddDeviceStyle() ' Create the new style and add it to the dictionary.
                EditPartData(oidStyle)                      ' Set some parameters in the style.
            Catch ex As System.Exception
                mEditor.WriteMessage(ex.Message)            ' We hit an exception; write out the error.
            End Try
        End Sub

#End Region

#Region "AddDeviceStyle"

        Public Function AddDeviceStyle() As ObjectId
            Using trans As Transaction = mTransMgr.StartTransaction()

                ' This is the object ID we'll return from this function; if we're successfull, this will
                ' be the ID of the new style we create, if not, it will be NULL.
                Dim oidReturn As ObjectId
                oidReturn = ObjectId.Null

                Dim dict As New DictionaryDeviceStyle(mDb)
                Dim style As New DeviceStyle()

                ' Our goal is to create a new device style from an existing block;
                ' this is the name of the existing block (it must be in the database already, that is,
                ' the drawing must already have a block defined as "MEP_block").
                Dim blockName As String = "MEP_block"

                ' Get the block table and make sure our block is in there.
                Dim bt As BlockTable
                bt = DirectCast(trans.GetObject(mDb.BlockTableId, OpenMode.ForRead), BlockTable)

                If Not bt.Has(blockName) Then
                    AcadApplication.ShowAlertDialog("Block not found")
                    Return ObjectId.Null
                End If

                ' We found our block, now create a new style from it.
                ' We're going to create an electrical device style using
                ' Lighting type and Geometry masking.
                Dim styleDescrip As String = "New Sample Style"
                style.Description = styleDescrip
                style.Type = MepType.Lighting
                style.Masking = Masking.Geometry

                ' We need to find the IDs for the Model and Plan display representations
                ' of a device. Initialize variables for use in the upcoming loop.
                Dim dr As DisplayRepresentation = Nothing
                Dim drIdModel As ObjectId = ObjectId.Null
                Dim drIdPlan As ObjectId = ObjectId.Null

                ' Get the collection of display representation IDs to search.
                Dim drm As New DisplayRepresentationManager(mDb)
                Dim devTmp As New Device()
                Dim rc As RXClass = devTmp.GetRXClass()
                Dim ids As ObjectIdCollection = drm.GetAllDisplayRepresentationsWorkForSpecifiedClass(rc)

                ' Go through the collection to find an ID for both the Model and Plan display
                ' representations (technically, we're finding the last ID for each of these
                ' display representations in the class).
                For Each drId As ObjectId In ids
                    dr = TryCast(trans.GetObject(drId, OpenMode.ForRead), DisplayRepresentation)

                    If dr.Name = "AecbDbDispRepDeviceModel" Then
                        drIdModel = dr.Id
                    End If

                    If dr.Name = "AecbDbDispRepDevicePlan" Then
                        drIdPlan = dr.Id
                    End If
                Next

                ' If we found an ID for the Model display representation, create a new
                ' multi-view part view definition (call it "Model") and add it to the
                ' multi-view part view definitions of our new style.
                If drIdModel <> ObjectId.Null Then
                    Dim mvpvd As New MultiViewPartViewDefinition()
                    mvpvd.AllViews = True
                    Dim blkid As ObjectId = bt(blockName)
                    mvpvd.BlockId = blkid
                    mvpvd.Name = "Model"
                    mvpvd.DisplayRepresentationId = drIdModel
                    style.MultiViewPartViewDefinitions.Add(mvpvd)
                End If

                ' If we found an ID for the Plan display representation, create a new
                ' multi-view part view definition (call it "Plan") and add it to the
                ' multi-view part view definitions of our new style.
                If drIdPlan <> ObjectId.Null Then
                    Dim mvpvd As New MultiViewPartViewDefinition()
                    mvpvd.AllViews = True
                    Dim blkid As ObjectId = bt(blockName)
                    mvpvd.BlockId = blkid
                    mvpvd.Name = "Plan"
                    mvpvd.DisplayRepresentationId = drIdPlan
                    style.MultiViewPartViewDefinitions.Add(mvpvd)
                End If

                ' Create a new multi-view part connector and add it to our new style (in this
                ' case, it's an electrical component connector of the Power-and-Lighting-System
                ' type).
                Dim mvpc As New MultiViewPartConnector()
                mvpc.Domain = Domain.ElectricalComponent
                mvpc.SystemType = SystemType.PowerAndLightingSystemType
                style.MultiViewPartConnectors.Add(mvpc)

                ' Add the Lighting classification to our style.
                Dim classDict As New DictionaryClassificationDefinition(mDb)

                Try
                    Dim classificationId As ObjectId = classDict.GetAt("Device Type")
                    Dim classDef As ClassificationDefinition = DirectCast(trans.GetObject(classificationId, OpenMode.ForRead), ClassificationDefinition)
                    Dim oid As ObjectId = classDef.GetClassification("Lighting")

                    If oid.IsValid Then
                        style.Classifications.Add(oid)
                    End If
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message) ' We hit an exception; write out the error.
                End Try

                ' Add our new style to the dictionary.
                Try
                    dict.AddNewRecord(styleDescrip, style)
                    mTransMgr.AddNewlyCreatedDBObject(style, True)
                    trans.Commit()
                    oidReturn = dict.GetAt(styleDescrip)
                Catch ex As System.Exception
                    mEditor.WriteMessage(ex.Message) ' We hit an exception; write out the error.
                End Try

                Return oidReturn
            End Using
        End Function

#End Region

#Region "EditPartData"

        Public Sub EditPartData(ByVal oidDevice As ObjectId)
            ' In this function, we'll edit a few parameters of the part data.
            Try
                Using trans As Transaction = mTransMgr.StartTransaction()
                    ' Open our style and get the data record.
                    Dim style As DeviceStyle = DirectCast(mTransMgr.GetObject(oidDevice, OpenMode.ForWrite, False), DeviceStyle)
                    Dim rec As DataRecord = PartManager.GetPartData(style)
                    Dim index As Integer = 1

                    ' Get the lighting demand factor style (it must already be defined in the drawing).
                    Dim demandDict As New DictionaryDemandFactorStyle(mDb)
                    Dim demandFactorStyle As ObjectId = demandDict.GetAt("Lighting")

                    For Each mvpConn As MultiViewPartConnector In style.MultiViewPartConnectors
                        ' Set the device load.
                        Dim dfLoad As New DataField(Context.ConnectionPropertyLoad, index)
                        dfLoad.ValueDouble = 600
                        rec.DataFields.Add(dfLoad)

                        ' Set the number of poles.
                        Dim dfPoles As New DataField(Context.ConnectionPropertyNoOfPoles, index)
                        dfPoles.ValueInteger = 1
                        rec.DataFields.Add(dfPoles)

                        ' Set the voltage.
                        Dim dfVoltage As New DataField(Context.ConnectionPropertyVoltage, index)
                        dfVoltage.ValueDouble = 120
                        rec.DataFields.Add(dfVoltage)

                        ' Set the demand.
                        If demandFactorStyle.IsValid Then
                            Dim dfDemand As New DataField(Context.ConnectionPropertyDemandFactor, index)
                            dfDemand.ValueObjectId = demandFactorStyle
                            rec.DataFields.Add(dfDemand)
                        End If

                        index += 1
                    Next

                    ' Add the layer key override.
                    Dim dfLayerKey As New DataField(Context.LayerKey, 0) ' This is a singleton data field, so index is zero.
                    dfLayerKey.ValueString = "E-DV-DEVICE"
                    rec.DataFields.Add(dfLayerKey)

                    ' Commit our changes.
                    PartManager.SetPartData(style, rec)
                    trans.Commit()
                End Using
            Catch ex As System.Exception
                mEditor.WriteMessage(ex.Message) ' We hit an exception; write out the error.
            End Try
        End Sub

#End Region

    End Class
End Namespace