#Region "Autodesk Copyright"

'      .NET Sample
'
'      Copyright (c) 2008 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
'

#End Region

#Region "Namespaces References"

Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports System.Windows.Forms
Imports System.Collections

' Excel specific
Imports System.Runtime.InteropServices ' For COMException  

' AutoCAD MEP Specific
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.GeomeTry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices

' create an alias to the AutoCAD Application object
Imports AcadApplication = Autodesk.AutoCAD.ApplicationServices.Application

#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(ExportToExcelCmd))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(ExportToExcelCmd))> 

Public Class ExportToExcelCmd
    Implements IExtensionApplication

#Region "IExtensionApplication"
    Public Sub Initialize() Implements IExtensionApplication.Initialize
        Dim ed As Editor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        ed.WriteMessage("Loading ExportToExcel..." & vbCrLf)
    End Sub

    Public Sub Terminate() Implements IExtensionApplication.Terminate
    End Sub
#End Region

#Region "Member Variables"
    Private mEditor As Editor
    Private mDatabase As Database
    Private mTransMgr As Autodesk.AutoCAD.DatabaseServices.TransactionManager
    Dim trans As Transaction
    Dim mExcelApp As Microsoft.Office.Interop.Excel.Application
    Dim m_ductsWorkSheet As Microsoft.Office.Interop.Excel.Worksheet
    Dim mWorkBook As Microsoft.Office.Interop.Excel.Workbook
    Dim mbHaveDuctHeaders As Boolean
    Dim mbHaveFittingHeaders As Boolean

    Dim mHeadList As ArrayList
#End Region

#Region "ExportDuctToExcel Command"

    ' <summary>
    ' Simple sample to demonstrate extracting data from an MEP part 
    ' </summary>
    <Autodesk.AutoCAD.Runtime.CommandMethod("ExportDuctToExcel", "ExportDuctToExcel", Autodesk.AutoCAD.Runtime.CommandFlags.Modal)> _
    Public Sub CmdExportDuctToExcel()

        mEditor = AcadApplication.DocumentManager.MdiActiveDocument.Editor
        mDatabase = AcadApplication.DocumentManager.MdiActiveDocument.Database
        mTransMgr = mDatabase.TransactionManager

        ' need to start the trans action before calling getMemberObject
        trans = mTransMgr.StartTransaction()

        Try
            ' prompt for a member object

            Dim oidSelected() As ObjectId
            oidSelected = GetMemberObjects("Select duct to export to Excel: ")

            ' export selected parts to Excel
            If oidSelected.Length > 0 Then
                SendToExcel(oidSelected)
            End If

            ' abort the transaction because we did not actually effect the database 
            trans.Abort()
        Catch e As System.Exception
            Debug.WriteLine(e.Message)
            trans.Abort()
        Finally
            trans.Dispose()
        End Try
    End Sub
#End Region

#Region "GetMemberObjects"

    ' <summary>
    ' Prompt for selecting the DUCT objects.
    ' </summary>
    ' <param name="prompt">The string to be displayed at the prompt line.</param>
    ' <returns>The collection of the duct object ids.</returns>
    Public Function GetMemberObjects(ByVal prompt As String) As Autodesk.AutoCAD.DatabaseServices.ObjectId()

        Try
            ' set up the entity prompt options, note that we only want a duct object
            Dim prOptions As PromptSelectionOptions = New PromptSelectionOptions()

            Dim values() As TypedValue = {New TypedValue(DxfCode.Start, "AECB_DUCT")}

            Dim selFilter As New SelectionFilter(values)

            ' prompt the user to select a duct
            Dim prResult As PromptSelectionResult = mEditor.GetSelection(prOptions, selFilter)

            If prResult.Status = PromptStatus.Cancel Then
                Return Nothing
            End If

            Dim selSet As SelectionSet = prResult.Value

            GetMemberObjects = selSet.GetObjectIds()

        Catch e As System.Exception
            Debug.Write(e.ToString())
            Throw e
        End Try
    End Function

#End Region

#Region "StartExcel"
    ' <summary>
    ' Start a new Excel application with a new workbook
    ' </summary>
    Private Sub StartExcel()
        ' Creates a new Excel Application
        '
        mExcelApp = New Microsoft.Office.Interop.Excel.ApplicationClass()
        mExcelApp.Visible = True

        ' The following line adds anew workbook
        '
        mWorkBook = mExcelApp.Workbooks.Add(System.Type.Missing)

    End Sub
#End Region

#Region "SendToExcel"

    ' <summary>
    ' Send the selected ducts to Excel for output.
    ' </summary>
    ' <param name="arrOids">The array of collected object ids.</param>
    Private Sub SendToExcel(ByVal arrOids As ObjectId())
        StartExcel()
        Dim ductRow As Integer = 2

        mbHaveDuctHeaders = False

        ' Iterate the objects to write out the properties in rows
        Dim oid As ObjectId
        For Each oid In arrOids
            Dim member As Member = CType(mTransMgr.GetObject(oid, OpenMode.ForRead, True), Member)
            If (member <> Nothing) Then
                Dim duct As Duct = member
                If (duct <> Nothing) Then
                    WritePartInfo(duct, ductRow)
                    ductRow = ductRow + 1
                End If
            End If
        Next

    End Sub

#End Region

#Region "WritePartInfo"

    ' <summary>
    ' Write out the part information.
    ' </summary>
    ' <param name="mem">The object to be written out.</param>
    ' <param name="row">The row number in Excel where the data shall go to.</param>
    Private Sub WritePartInfo(ByVal mem As Member, ByVal row As Integer)

        Try

            If Not mem Is Nothing Then

                Dim duct As Duct = mem

                ' Write out the header row only once. 
                If duct <> Nothing Then

                    If mbHaveDuctHeaders = False Then
                        m_ductsWorkSheet = mWorkBook.Sheets.Add()
                        m_ductsWorkSheet.Name = "Ducts"
                        mbHaveDuctHeaders = True
                        WriteHeaders(duct)
                    End If

                End If

                ' Write out the fixed set of common properties.
                Dim col As Integer = WriteProperties(duct, row)

                ' Write out the part data. Note that different object may have different data fields.
                Dim rec As DataRecord = PartManager.GetPartData(mem)

                Dim df As DataField

                For Each df In rec.DataFields

                    Dim cell As Microsoft.Office.Interop.Excel.Range = m_ductsWorkSheet.Cells(row, col)

                    Select Case df.Type
                        Case CatalogDataType.Bool
                            cell.Value2 = df.ValueBoolean.ToString()
                        Case CatalogDataType.Double
                            cell.Value2 = df.ValueDouble.ToString()
                        Case CatalogDataType.Int
                            cell.Value2 = df.ValueInteger.ToString()
                        Case CatalogDataType.String
                            cell.Value2 = df.ValueString.ToString()
                        Case Else
                            cell.Value2 = df.Type.ToString()
                    End Select

                    cell.EntireColumn.AutoFit()

                    col = col + 1
                Next

            End If

        Catch comE As COMException
            MessageBox.Show(comE.Message)

        Catch e As System.Exception
            MessageBox.Show(e.ToString())

        End Try

    End Sub

#End Region

#Region "WriteHeaders"

    ' <summary>
    ' Write the header row for the duct objects. Note that there is no logic for different 
    ' ducts that may contain different DataField
    ' </summary>
    ' <param name="member">The object to be written out.</param>
    ' <param name="bIsDuct">The flag to indicate if the object is a duct or not.</param>
    Private Sub WriteHeaders(ByVal member As Member)

        Try
            Dim col As Integer
            col = WritePropertiesHeaders()

            ' now get the part data record from the part
            '
            Dim rec As DataRecord = PartManager.GetPartData(member)
            Dim cell As Microsoft.Office.Interop.Excel.Range

            Dim df As DataField
            For Each df In rec.DataFields
                cell = m_ductsWorkSheet.Cells(1, col)
                cell.Value2 = df.Description
                cell.Font.Bold = True
                col = col + 1
            Next
        Catch e As System.Exception
            MessageBox.Show(e.ToString())

        End Try
    End Sub

#End Region

#Region "WriteProperties"

    ' <summary>
    ' Write the fixed set of common propreties (layer, start point, etc...).
    ' </summary>
    ' <param name="memberObj">The object to be written out.</param>
    ' <param name="row">The row number in Excel where the data shall go to.</param>
    ' <returns>The column number of the next available cell.</returns>
    Function WriteProperties(ByRef memberObj As Member, ByVal row As Integer) As Integer

        Dim cell As Microsoft.Office.Interop.Excel.Range

        Dim col As Integer = 1

        ' layer
        Dim layerId As ObjectId = memberObj.LayerId
        Dim ltr As LayerTableRecord = CType(mTransMgr.GetObject(layerId, OpenMode.ForRead, True), LayerTableRecord)
        cell = m_ductsWorkSheet.Cells(row, col)
        cell.Value2 = ltr.Name
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' start point, sample formatted
        cell = m_ductsWorkSheet.Cells(row, col)
        Dim X As String, Y As String, Z As String
        X = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(memberObj.StartPoint.X)
        Y = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(memberObj.StartPoint.Y)
        Z = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(memberObj.StartPoint.Z)

        cell.Value2 = X + "," + Y + "," + Z
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' end point, sample unformatted
        cell = m_ductsWorkSheet.Cells(row, col)
        cell.Value2 = memberObj.EndPoint.ToString()
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' length, formatted
        Dim length As Double = memberObj.StartPoint.DistanceTo(memberObj.EndPoint)
        cell = m_ductsWorkSheet.Cells(row, col)
        cell.Value2 = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(memberObj.StartPoint.DistanceTo(memberObj.EndPoint))
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' cut length, formatted
        Dim ducttmp As Duct = memberObj
        If (ducttmp <> Nothing) Then
            cell = m_ductsWorkSheet.Cells(row, col)
            cell.Value2 = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(ducttmp.CutLength)
            cell.EntireColumn.AutoFit()
            col = col + 1
        End If

        ' normal
        cell = m_ductsWorkSheet.Cells(row, col)
        cell.Value2 = memberObj.Normal.ToString()
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' elevation
        Dim strelev As String
        If memberObj.StartPoint.Z = memberObj.EndPoint.Z Then
            strelev = memberObj.StartPoint.Z.ToString()
        Else
            strelev = "Sloped"
        End If

        cell = m_ductsWorkSheet.Cells(row, col)
        cell.Value2 = strelev
        cell.EntireColumn.AutoFit()
        col = col + 1

        ' insulation thickness
        cell = m_ductsWorkSheet.Cells(row, col)
        Dim strInsulation As String = Autodesk.AutoCAD.Runtime.Converter.DistanceToString(ducttmp.InsulationThickness)
        cell.Value2 = strInsulation

        cell.EntireColumn.AutoFit()

        col = col + 1

        Return col

    End Function

#End Region

#Region "WritePropertiesHeaders"
    ' <summary>
    ' Write out the header row for the fixed set of common properties.
    ' </summary>
    ' <returns>The column number of the next available cell.</returns>
    Function WritePropertiesHeaders() As Integer

        Dim cell As Microsoft.Office.Interop.Excel.Range
        Dim col As Integer = 1

        ' layer
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Layer"
        cell.Font.Bold = True
        col = col + 1

        ' start point
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Start Point"
        cell.Font.Bold = True
        col = col + 1

        ' end point
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "End Point"
        cell.Font.Bold = True
        col = col + 1

        ' length
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Length"
        cell.Font.Bold = True
        col = col + 1

        ' cut length
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Cut Length"
        cell.Font.Bold = True
        col = col + 1

        ' normal 
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Normal"
        cell.Font.Bold = True
        col = col + 1

        ' elevation
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Elevation"
        cell.Font.Bold = True
        col = col + 1

        ' insulation thickness
        cell = m_ductsWorkSheet.Cells(1, col)
        cell.Value2 = "Insulation Thickness"
        cell.Font.Bold = True
        col = col + 1

        Return col

    End Function

#End Region

End Class



