#Region "Autodesk Copyright "
'
'  .NET Sample
'
'  Copyright (c) 2007 by Autodesk, Inc.
'
'  Permission to use, copy, modify, and distribute this software
'  for any purpose and without fee is hereby granted, provided
'  that the above copyright notice appears in all copies and
'  that both that copyright notice and the limited warranty and
'  restricted rights notice below appear in all supporting
'  documentation.
'
'  AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'  AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'  UNINTERRUPTED OR ERROR FREE.
'
'  Use, duplication, or disclosure by the U.S. Government is subject to
'  restrictions set forth in FAR 52.227-19 (Commercial Computer
'  Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'  (Rights in Technical Data and Computer Software), as applicable.
'
#End Region
#Region "References"
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Diagnostics
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.Aec.Building.ApplicationServices
Imports Autodesk.Aec.Building.DatabaseServices
Imports Autodesk.Aec.Building.Hvac.DatabaseServices
Imports Autodesk.Aec.Building.Elec.DatabaseServices
Imports Autodesk.Aec.Building.Piping.DatabaseServices
Imports Autodesk.Aec.Building.Plumbing.DatabaseServices
#End Region

<Assembly: Autodesk.AutoCAD.Runtime.ExtensionApplication(GetType(System.DBNull))> 
<Assembly: Autodesk.AutoCAD.Runtime.CommandClass(GetType(System.DBNull))> 

Namespace MepUtilities
    Public Class DataUtilities
#Region "DataFieldToString"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.DataFieldToString </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' This method returns a string representing the data that is
        ''' stored in the datafield</summary>
        ''' <param name="datafield"></param>
        ''' </member>

        Public Shared Function DataFieldToString(ByVal datafield As DataField) As String
            Dim value As String
            Select Case datafield.Type
                Case CatalogDataType.Bool
                    value = datafield.ValueBoolean.ToString
                    ' break
                Case CatalogDataType.Double
                    value = datafield.ValueDouble.ToString
                    ' break
                Case CatalogDataType.Int
                    value = datafield.ValueInteger.ToString
                    ' break
                Case CatalogDataType.String
                    value = datafield.ValueString
                    ' break
                Case CatalogDataType.ObjectId
                    value = datafield.ValueObjectId.ToString
                    ' break
                Case Else
                    value = datafield.Type.ToString
                    ' break
            End Select
            Return value
        End Function

#End Region

#Region "AddDataFieldToQuery"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.AddDataFieldToQuery </name>
        ''' <returntype> void </returntype>
        ''' <summary>
        ''' This method adds the given datafield to the query.  It adds
        ''' an approprate size parameter for the type of data contained
        ''' in the data field</summary>
        ''' <param name="datafield"></param>
        ''' <param name="query"></param>
        ''' </member>

        Public Shared Sub AddDataFieldToQuery(ByVal datafield As DataField, ByRef query As DataQuery)
            Select Case datafield.Type
                Case CatalogDataType.Bool
                    query.AddSizeParameter(datafield.Context, datafield.Index, datafield.ValueBoolean)
                    ' break
                Case CatalogDataType.Double
                    query.AddSizeParameter(datafield.Context, datafield.Index, datafield.ValueDouble)
                    ' break
                Case CatalogDataType.Int
                    query.AddSizeParameter(datafield.Context, datafield.Index, datafield.ValueInteger)
                    ' break
                Case CatalogDataType.String
                    query.AddSizeParameter(datafield.Context, datafield.Index, datafield.ValueString)
                    ' break
                Case Else
                    query.AddSizeParameter(datafield.Context, datafield.Index)
                    ' break
            End Select
        End Sub

#End Region

#Region "GetPartDomain"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartDomain </name>
        ''' <returntype> Autodesk.Aec.Building.DatabaseServices.Domain </returntype>
        ''' <summary>
        ''' Gets the domain from the given data record</summary>
        ''' <param name="dataRecord"></param>
        ''' </member>

        Public Shared Function GetPartDomain(ByVal dataRecord As DataRecord) As Domain
            Dim value As Domain
            value = Domain.UndefinedDomain

            ' Get the name of the domain,  as a string
            Dim sDomain As String
            Try
                sDomain = getStringValue(dataRecord, Context.CatalogDomain)
            Catch ex As BuildingException
                Throw ex
            End Try

            ' Convert the name of the domain into the enum
            Dim nDomain As Integer
            nDomain = Autodesk.Aec.Building.DatabaseServices.Utility.GetEnumFromGlobalString(sDomain, value.GetType())

            value = CType(nDomain, Domain)
            Return value

        End Function

#End Region

#Region "GetPartType"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartType </name>
        ''' <returntype> Autodesk.Aec.Building.DatabaseServices.Type </returntype>
        ''' <summary>
        ''' Gets the part type from the given data record.
        ''' </summary>
        ''' <param name="dataRecord"></param>
        ''' </member>

        Public Shared Function GetPartType(ByVal dataRecord As DataRecord) As Autodesk.Aec.Building.DatabaseServices.Type
            Dim value As Autodesk.Aec.Building.DatabaseServices.Type
            value = Autodesk.Aec.Building.DatabaseServices.Type.UndefinedType

            ' Get the name of the type,  as a string
            Dim sType As String
            Try
                sType = getStringValue(dataRecord, Context.CatalogType)
            Catch ex As BuildingException
                Throw ex
            End Try

            ' Convert the name of the type into an enum
            Dim nType As Integer
            nType = Autodesk.Aec.Building.DatabaseServices.Utility.GetEnumFromGlobalString(sType, value.GetType())

            value = CType(nType, Autodesk.Aec.Building.DatabaseServices.Type)
            Return value

        End Function

#End Region

#Region "GetPartName"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartName </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Gets the part name of the given member derived object.
        ''' </summary>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartName(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the part name from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogPartName)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartSizeName"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartSizeName </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Gets the part size name from the given data record.
        ''' </summary>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartSizeName(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the part size name from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogPartSizeName)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartDescription"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartDescription </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Get the part description from the given data record
        ''' </summary>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartDescription(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the part description from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogPartDescription)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartCatalogGuid"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartCatalogGuid </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Get the catalog guid from the given data record
        ''' </summary>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartCatalogGuid(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the catalog guid from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogId)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartGuid"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartGuid </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Gets the part guid from the given data record
        ''' </summary>
        ''' <returns> The part guid from the given data record </returns>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartGuid(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the part guid from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogPartId)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartBasicTableRowId"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartBasicTableRowId </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Gets the basic table row id from the given data record
        ''' </summary>
        ''' <returns> The basic table row id from the given data record </returns>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartBasicTableRowId(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the basic table row id from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogBasicTableRowId)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "GetPartSubType"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.GetPartSubType </name>
        ''' <returntype> string </returntype>
        ''' <summary>
        ''' Gets the subtype from the given data record
        ''' </summary>
        ''' <returns> The sub-type from the given data record </returns>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function GetPartSubType(ByVal dataRecord As DataRecord) As String
            Dim value As String
            ' Get the part sub-type from the data record
            Try
                value = getStringValue(dataRecord, Context.CatalogSubType)
            Catch ex As BuildingException
                Throw ex
            End Try
            Return value
        End Function

#End Region

#Region "IsPartPlaceholder"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.IsPartPlaceholder </name>
        ''' <returntype> bool </returntype>
        ''' <summary>
        ''' Gets the IsPlaceHolder value from the given data record
        ''' </summary>
        ''' <returns> True if the record is for a placeholder part,  otherwise false </returns>
        ''' <param name="dataRecord"></param>
        ''' </member>


        Public Shared Function IsPartPlaceholder(ByVal dataRecord As DataRecord) As Boolean
            Dim bIsPlaceHolder As Boolean = False
            Dim datafield As DataField = dataRecord.DataFields.FindByContext(Context.PartPlaceholder)
            If Not (datafield Is Nothing) Then
                bIsPlaceHolder = datafield.ValueBoolean
            Else
                bIsPlaceHolder = False
            End If

            Return bIsPlaceHolder
        End Function

#End Region

#Region "getStringValue"

        ''' <member>
        ''' <name> MepUtilities.DataUtilities.getStringValue </name>
        ''' <returntype> string </returntype>
        ''' <summary>Returns the string value of the data field of the given context
        ''' from the given data record</summary>
        ''' <returns>The string value from the data field</returns>        
        ''' <param name="record"></param>
        ''' <param name="eContext"></param>
        ''' </member>

        Private Shared Function getStringValue(ByVal record As DataRecord, ByVal eContext As Context) As String
            Dim value As String

            Dim dataField As DataField = record.DataFields.FindByContext(eContext)
            If Not (dataField Is Nothing) Then
                value = dataField.ValueString
            Else
                Throw New Autodesk.Aec.Building.DatabaseServices.BuildingException(Autodesk.Aec.Building.DatabaseServices.ErrorCode.QueryFailed)
            End If

            Return value
        End Function

#End Region

    End Class
End Namespace

